// src/lib/stores.ts
import { writable, type Writable } from 'svelte/store';

// --- Re-declare or Import Types ---
// (Ensure these match the definitions in your components)
interface User {
	id: string;
	initials: string;
	name: string;
	color?: string;
}

interface ShoppingItem {
	id: string;
	text: string;
	completed: boolean;
	// isSwiped is view-specific, likely doesn't belong in the store model
}

// Chore model for store (without isSwiped)
interface Chore {
	id: string;
	text: string;
	completed: boolean;
	assignedTo: User | null;
	category: string;
}
// Helper for storing chores (maps User object to ID)
interface StorableChore extends Omit<Chore, 'assignedTo'> {
	assignedToId: string | null;
}

// Group model for store
interface Group {
	id: string;
	name: string;
	members: User[];
}
// Helper for storing groups (maps User array to ID array)
interface StorableGroup {
	id: string;
	name: string;
	memberIds: string[];
}

// --- Helper: Get data from LocalStorage ---
function loadFromLocalStorage<T>(
	key: string,
	defaultValue: T,
	reviver?: (key: string, value: unknown) => unknown
): T {
	if (typeof window === 'undefined') {
		return defaultValue; // Cannot access localStorage on server
	}
	const saved = localStorage.getItem(key);
	if (saved) {
		try {
			return JSON.parse(saved, reviver);
		} catch (e) {
			console.error(`Failed to parse ${key} from localStorage`, e);
			return defaultValue;
		}
	}
	return defaultValue;
}

// --- Helper: Persist store data to LocalStorage ---
function persistStore<T>(
	store: Writable<T>,
	key: string,
	replacer?: (key: string, value: unknown) => unknown
) {
	if (typeof window === 'undefined') return; // Skip persistence on server

	// Update localStorage on store change
	store.subscribe((value) => {
		localStorage.setItem(key, JSON.stringify(value, replacer));
	});
}

// --- Static User Data (Should ideally come from auth/API) ---
// (Keep this consistent with other components or centralize it)
const availableUsers: User[] = [
	{ id: 'user1', initials: 'JD', name: 'Jane Doe', color: '#ffb26b' },
	{ id: 'user2', initials: 'AP', name: 'Alex P.', color: '#a0e7a0' },
	{ id: 'user3', initials: 'MS', name: 'Mike Smith', color: '#54c7ff' },
	{ id: 'user4', initials: 'LS', name: 'Lisa Simpson', color: '#ffd56b' }
];

// --- Stores ---

// Shopping List Store
const initialShoppingItems = loadFromLocalStorage<ShoppingItem[]>('shoppingListItems', []);
export const shoppingItemsStore = writable<ShoppingItem[]>(initialShoppingItems);
persistStore(shoppingItemsStore, 'shoppingListItems');

// Chores Store (with revival logic)
const loadedStorableChores = loadFromLocalStorage<StorableChore[]>('choresList', []);
const initialChores = loadedStorableChores.map((sc) => ({
	id: sc.id,
	text: sc.text,
	completed: sc.completed,
	category: sc.category,
	assignedTo: sc.assignedToId ? availableUsers.find((u) => u.id === sc.assignedToId) || null : null
	// isSwiped is intentionally omitted from the store model
}));
export const choresStore = writable<Chore[]>(initialChores);
// Persist helper (maps User back to ID)
persistStore(choresStore, 'choresList', (key, value) => {
	if (key === 'assignedTo' && value) {
		return (value as User).id; // Store only the ID
	}
	return value;
});
// Need custom mapping logic when saving, standard replacer isn't enough.
// Better approach for complex objects: manually stringify in subscribe
choresStore.subscribe((chores) => {
	if (typeof window !== 'undefined') {
		const storableChores = chores.map((c) => ({
			...c,
			assignedToId: c.assignedTo ? c.assignedTo.id : null,
			assignedTo: undefined // Remove object before saving
		}));
		localStorage.setItem('choresList', JSON.stringify(storableChores));
	}
});

// Groups Store (with revival logic)
const loadedStorableGroups = loadFromLocalStorage<StorableGroup[]>('groupsList', []);
const initialGroups = loadedStorableGroups.map((sg) => ({
	id: sg.id,
	name: sg.name,
	members: sg.memberIds
		.map((id) => availableUsers.find((u) => u.id === id))
		.filter((user) => user !== undefined) as User[]
}));
export const groupsStore = writable<Group[]>(initialGroups);
// Persist helper (maps User array back to ID array)
groupsStore.subscribe((groups) => {
	if (typeof window !== 'undefined') {
		const storableGroups = groups.map((g) => ({
			id: g.id,
			name: g.name,
			memberIds: g.members.map((m) => m.id)
		}));
		localStorage.setItem('groupsList', JSON.stringify(storableGroups));
	}
});

// You might also want a store for the current user if you have authentication
// export const currentUserStore = writable<User | null>(null);
