import { supabase } from './supabaseClient';

// --- Define aliases for easier use ---
type DbShoppingItemInsert = Database['public']['Tables']['shopping_items']['Insert'];
type DbShoppingItemUpdate = Database['public']['Tables']['shopping_items']['Update'];
type DbShoppingItem = Database['public']['Tables']['shopping_items']['Row'];

type DbChoreInsert = Database['public']['Tables']['chores']['Insert'];
type DbChoreUpdate = Database['public']['Tables']['chores']['Update'];
type DbChore = Database['public']['Tables']['chores']['Row'];
type DbProfile = Database['public']['Tables']['profiles']['Row']; // Assuming profiles table

type DbGroupInsert = Database['public']['Tables']['groups']['Insert'];
type DbGroupUpdate = Database['public']['Tables']['groups']['Update'];
type DbGroup = Database['public']['Tables']['groups']['Row'];
type DbGroupMemberInsert = Database['public']['Tables']['group_members']['Insert'];

// Combined type for Chore with Profile data
export interface ChoreWithProfile extends Omit<DbChore, 'assigned_to_user_id'> {
	assigned_to: DbProfile | null; // Replace FK with nested profile object
}

// Combined type for Group with Members (including profile data)
export interface GroupWithMembers extends DbGroup {
	members: DbProfile[];
}

// === Error Handling Helper ===
async function handleSupabaseResponse<T>(response: { data: T | null; error: any }): Promise<T> {
	if (response.error) {
		console.error('Supabase Error:', response.error.message);
		throw new Error(response.error.message); // Re-throw for service layer handling
	}
	if (response.data === null) {
		// This might happen on single() select if not found, or other cases
		throw new Error('No data returned from Supabase.');
	}
	return response.data;
}
async function handleSupabaseVoidResponse(response: { error: any }): Promise<void> {
	if (response.error) {
		console.error('Supabase Error:', response.error.message);
		throw new Error(response.error.message);
	}
}

// === Shopping List Service ===

export const shoppingListService = {
	async getAll(): Promise<DbShoppingItem[]> {
		// Assumes RLS handles user filtering if needed
		const response = await supabase
			.from('shopping_items')
			.select('*')
			.order('created_at', { ascending: false });
		return handleSupabaseResponse(response);
	},

	async add(itemData: Pick<DbShoppingItemInsert, 'text'>): Promise<DbShoppingItem> {
		// Add user_id here if your schema requires it, get from auth state
		const response = await supabase
			.from('shopping_items')
			.insert({ text: itemData.text /*, user_id: userId */ })
			.select()
			.single();
		return handleSupabaseResponse(response);
	},

	async update(id: string, updates: DbShoppingItemUpdate): Promise<DbShoppingItem> {
		const response = await supabase
			.from('shopping_items')
			.update(updates)
			.eq('id', id)
			.select()
			.single();
		return handleSupabaseResponse(response);
	},

	async toggleComplete(id: string, completed: boolean): Promise<DbShoppingItem> {
		return this.update(id, { completed });
	},

	async delete(id: string): Promise<void> {
		const response = await supabase.from('shopping_items').delete().eq('id', id);
		return handleSupabaseVoidResponse(response);
	}
};

// === Chores Service ===

export const choreService = {
	async getAll(): Promise<ChoreWithProfile[]> {
		// Fetch chores and join the assigned profile data
		const response = await supabase
			.from('chores')
			.select(
				`
                *,
                assigned_to:profiles!assigned_to_user_id (*)
            `
			)
			.order('created_at', { ascending: false });

		// Need to handle the structured response
		if (response.error) {
			console.error('Supabase Error:', response.error.message);
			throw new Error(response.error.message);
		}
		// Type assertion needed because Supabase type gen might not perfectly match nested select
		return ((response.data as any[]) || []) as ChoreWithProfile[];
	},

	async add(choreData: {
		text: string;
		category: string;
		assignedToUserId: string | null;
	}): Promise<ChoreWithProfile> {
		const response = await supabase
			.from('chores')
			.insert({
				text: choreData.text,
				category: choreData.category,
				assigned_to_user_id: choreData.assignedToUserId
			})
			.select(`*, assigned_to:profiles!assigned_to_user_id (*)`)
			.single();
		// Type assertion needed
		return handleSupabaseResponse(response as any) as ChoreWithProfile;
	},

	async update(
		id: string,
		updates: {
			text?: string;
			category?: string;
			assignedToUserId?: string | null;
			completed?: boolean;
		}
	): Promise<ChoreWithProfile> {
		// Prepare update object, mapping assignedToUserId
		const dbUpdates: DbChoreUpdate = {
			...(updates.text && { text: updates.text }),
			...(updates.category && { category: updates.category }),
			...(updates.completed !== undefined && { completed: updates.completed }),
			// Handle null explicitly for foreign keys if needed
			assigned_to_user_id:
				updates.assignedToUserId === undefined ? undefined : updates.assignedToUserId
		};

		const response = await supabase
			.from('chores')
			.update(dbUpdates)
			.eq('id', id)
			.select(`*, assigned_to:profiles!assigned_to_user_id (*)`)
			.single();
		// Type assertion needed
		return handleSupabaseResponse(response as any) as ChoreWithProfile;
	},

	async toggleComplete(id: string, completed: boolean): Promise<ChoreWithProfile> {
		return this.update(id, { completed });
	},

	async delete(id: string): Promise<void> {
		const response = await supabase.from('chores').delete().eq('id', id);
		return handleSupabaseVoidResponse(response);
	}
};

// === Groups Service ===

export const groupService = {
	async getAll(): Promise<GroupWithMembers[]> {
		// Fetch groups and join members via the join table, then join profiles
		const response = await supabase
			.from('groups')
			.select(
				`
                *,
                group_members (
                    user_id,
                    profiles (*)
                )
            `
			)
			.order('name');

		if (response.error) {
			console.error('Supabase Error:', response.error.message);
			throw new Error(response.error.message);
		}

		// Manually map the nested structure to the desired GroupWithMembers[] format
		const groups = (response.data || []).map((group: any) => ({
			...group,
			members: group.group_members // Access nested data
				.map((gm: any) => gm.profiles) // Extract the profile from each member entry
				.filter((profile: any) => profile !== null) // Filter out potential null profiles if data inconsistent
		}));

		return groups as GroupWithMembers[];
	},

	// Adding a group with members requires multiple steps
	async add(groupData: { name: string; memberIds: string[] }): Promise<GroupWithMembers> {
		// 1. Insert the group
		const groupResponse = await supabase
			.from('groups')
			.insert({ name: groupData.name })
			.select()
			.single();
		const newGroup = await handleSupabaseResponse(groupResponse);

		// 2. If successful and members exist, insert into join table
		if (newGroup && groupData.memberIds.length > 0) {
			const membersToInsert: DbGroupMemberInsert[] = groupData.memberIds.map((userId) => ({
				group_id: newGroup.id,
				user_id: userId
			}));
			const membersResponse = await supabase.from('group_members').insert(membersToInsert);
			// Check for errors on member insert, but proceed to return group anyway for now
			if (membersResponse.error) {
				console.error('Supabase Error adding members:', membersResponse.error.message);
				// Decide how to handle partial failure: throw error? log and continue?
				// For simplicity, we'll continue and return the group, members might be incomplete
			}
		}

		// 3. Fetch the newly created group with its members to return consistent type
		return this.getById(newGroup.id); // Assumes getById exists (implement below)
	},

	// Updating involves updating name and managing member changes (add/remove)
	async update(
		id: string,
		updates: { name?: string; memberIds?: string[] }
	): Promise<GroupWithMembers> {
		// 1. Update group name if provided
		if (updates.name) {
			const nameUpdateResponse = await supabase
				.from('groups')
				.update({ name: updates.name })
				.eq('id', id);
			await handleSupabaseVoidResponse(nameUpdateResponse); // Check for error
		}

		// 2. Handle member updates if provided
		if (updates.memberIds) {
			const newMemberIds = updates.memberIds;
			// Get current members
			const currentMembersResponse = await supabase
				.from('group_members')
				.select('user_id')
				.eq('group_id', id);
			const currentMemberData = await handleSupabaseResponse(currentMembersResponse);
			const currentMemberIds = currentMemberData.map((m) => m.user_id);

			// Calculate differences
			const membersToAdd = newMemberIds.filter((userId) => !currentMemberIds.includes(userId));
			const membersToRemove = currentMemberIds.filter((userId) => !newMemberIds.includes(userId));

			// Add new members
			if (membersToAdd.length > 0) {
				const membersToInsert: DbGroupMemberInsert[] = membersToAdd.map((userId) => ({
					group_id: id,
					user_id: userId
				}));
				const addResponse = await supabase.from('group_members').insert(membersToInsert);
				await handleSupabaseVoidResponse(addResponse);
			}

			// Remove old members
			if (membersToRemove.length > 0) {
				const removeResponse = await supabase
					.from('group_members')
					.delete()
					.eq('group_id', id)
					.in('user_id', membersToRemove);
				await handleSupabaseVoidResponse(removeResponse);
			}
		}

		// 3. Fetch the updated group with members to return
		return this.getById(id);
	},

	async delete(id: string): Promise<void> {
		// 1. Delete memberships first (due to foreign key constraints)
		const membersDeleteResponse = await supabase.from('group_members').delete().eq('group_id', id);
		await handleSupabaseVoidResponse(membersDeleteResponse); // Check error

		// 2. Delete the group itself
		const groupDeleteResponse = await supabase.from('groups').delete().eq('id', id);
		await handleSupabaseVoidResponse(groupDeleteResponse); // Check error
	},

	// Helper to get a single group with members (used after add/update)
	async getById(id: string): Promise<GroupWithMembers> {
		const response = await supabase
			.from('groups')
			.select(`*, group_members (user_id, profiles (*))`)
			.eq('id', id)
			.single(); // Expect only one

		if (response.error) {
			console.error('Supabase Error:', response.error.message);
			throw new Error(response.error.message);
		}
		if (!response.data) {
			throw new Error(`Group with id ${id} not found.`);
		}

		const group: any = response.data;
		const result: GroupWithMembers = {
			...group,
			members: group.group_members
				.map((gm: any) => gm.profiles)
				.filter((profile: any) => profile !== null)
		};
		return result;
	}
};

// === Profile/User Service ===

export const profileService = {
	// Gets all profiles - useful for assigning users
	async getAll(): Promise<DbProfile[]> {
		const response = await supabase.from('profiles').select('*').order('name');
		return handleSupabaseResponse(response);
	},

	// Gets the profile for the currently logged-in user
	async getCurrentUserProfile(): Promise<DbProfile | null> {
		const {
			data: { user },
			error: authError
		} = await supabase.auth.getUser();

		if (authError) {
			console.error('Auth Error getting user:', authError.message);
			return null; // Not logged in or error
		}
		if (!user) {
			return null; // Should not happen if error is null, but check anyway
		}

		const response = await supabase.from('profiles').select('*').eq('id', user.id).single();

		// Don't throw error if profile not found, just return null
		if (response.error && response.error.code !== 'PGRST116') {
			// PGRST116 = Range not satisfiable (expected for single() with no result)
			console.error('Supabase Error fetching profile:', response.error.message);
			throw new Error(response.error.message);
		}

		return response.data;
	},

	// Add function to update profile if needed (name, initials, color)
	async updateCurrentProfile(
		updates: Partial<Pick<DbProfile, 'name' | 'initials' | 'color'>>
	): Promise<DbProfile> {
		const {
			data: { user },
			error: authError
		} = await supabase.auth.getUser();
		if (authError || !user) throw new Error('User not authenticated.');

		const response = await supabase
			.from('profiles')
			.update(updates)
			.eq('id', user.id)
			.select()
			.single();
		return handleSupabaseResponse(response);
	}
};
