// src/lib/components/types.ts

// --- Enum Type Definitions (Inferred) ---
// Replace with your actual enum values if different

export type ChoreStatus = 'pending' | 'completed' | 'skipped' | 'overdue'; // Example values
export type ChoreFrequency = 'daily' | 'weekly' | 'monthly' | 'yearly'; // Example values
export type SplitType = 'equal' | 'exact' | 'percentage'; // Example values
export type SettlementActivityType = 'payment' | 'refund' | 'adjustment' | 'marked_paid'; // Example values
export type UserRole = 'owner' | 'admin' | 'member'; // Example values

// --- Table Interface Definitions ---

/**
 * Corresponds to the 'public.chore_assignments' table
 */
export interface ChoreAssignment {
	id: number;
	chore_id: number;
	assigned_user_id: string | null; // uuid
	due_date: string; // date (ISO 8601 format: YYYY-MM-DD)
	completion_status: ChoreStatus;
	completed_at: string | null; // timestamp with time zone (ISO 8601 format)
	assigned_at: string; // timestamp with time zone (ISO 8601 format)
	updated_at: string; // timestamp with time zone (ISO 8601 format)
}

/**
 * Corresponds to the 'public.chores' table
 */
export interface Chore {
	id: number;
	group_id: number;
	name: string;
	description: string | null;
	created_by_id: string; // uuid
	created_at: string; // timestamp with time zone (ISO 8601 format)
	updated_at: string; // timestamp with time zone (ISO 8601 format)
	is_recurring: boolean;
	frequency_type: ChoreFrequency | null;
	frequency_value: number | null;
	next_due_date: string | null; // date (ISO 8601 format: YYYY-MM-DD)
	last_completed_at: string | null; // timestamp with time zone (ISO 8601 format)
	is_active: boolean;
}

/**
 * Corresponds to the 'public.expense_records' table
 */
export interface ExpenseRecord {
	id: number;
	list_id: number;
	calculated_at: string; // timestamp with time zone (ISO 8601 format)
	calculated_by_id: string; // uuid
	total_amount: number; // numeric(10, 2) - use number for simplicity, consider string for precision if needed
	participants: string[]; // uuid[]
	split_type: SplitType;
	is_settled: boolean;
}

/**
 * Corresponds to the 'public.expense_shares' table
 */
export interface ExpenseShare {
	id: number;
	expense_record_id: number;
	user_id: string; // uuid
	amount_owed: number; // numeric(10, 2) - use number for simplicity, consider string for precision if needed
	is_paid: boolean;
}

/**
 * Corresponds to the 'public.groups' table
 */
export interface Group {
	id: number;
	name: string;
	created_by_id: string; // uuid
	created_at: string; // timestamp with time zone (ISO 8601 format)
}

/**
 * Corresponds to the 'public.invites' table
 */
export interface Invite {
	id: number;
	code: string;
	group_id: number;
	created_by_id: string; // uuid
	created_at: string; // timestamp with time zone (ISO 8601 format)
	expires_at: string; // timestamp with time zone (ISO 8601 format)
	is_active: boolean;
}

/**
 * Corresponds to the 'public.items' table
 */
export interface Item {
	id: number;
	list_id: number;
	name: string;
	quantity: string | null;
	is_complete: boolean;
	price: number | null; // numeric(10, 2) - use number for simplicity, consider string for precision if needed
	price_added_by_id: string | null; // uuid
	price_added_at: string | null; // timestamp with time zone (ISO 8601 format)
	added_by_id: string; // uuid
	completed_by_id: string | null; // uuid
	created_at: string; // timestamp with time zone (ISO 8601 format)
	updated_at: string; // timestamp with time zone (ISO 8601 format)
}

/**
 * Corresponds to the 'public.lists' table
 * Added optional 'items' for card display example
 */
export interface List {
	id: number;
	name: string;
	description: string | null;
	created_by_id: string; // uuid
	group_id: number | null;
	is_complete: boolean;
	created_at: string; // timestamp with time zone (ISO 8601 format)
	updated_at: string; // timestamp with time zone (ISO 8601 format)
	items?: Item[]; // Optional: Include items if needed for card display
}

/**
 * Corresponds to the 'public.profiles' table
 * Often linked to auth.users
 */
export interface Profile {
	id: string; // uuid (usually matches auth.users.id)
	name: string | null;
	created_at: string; // timestamp with time zone (ISO 8601 format)
	updated_at: string; // timestamp with time zone (ISO 8601 format)
	// You might add other profile fields here like avatar_url, etc.
}

/**
 * Corresponds to the 'public.settlement_activities' table
 */
export interface SettlementActivity {
	id: number;
	expense_record_id: number;
	payer_user_id: string; // uuid
	affected_user_id: string; // uuid
	activity_type: SettlementActivityType;
	timestamp: string; // timestamp with time zone (ISO 8601 format)
}

/**
 * Corresponds to the 'public.user_groups' table
 * Represents the relationship between users and groups (membership)
 */
export interface UserGroup {
	id: number;
	user_id: string; // uuid
	group_id: number;
	role: UserRole;
	joined_at: string; // timestamp with time zone (ISO 8601 format)
}

// --- Optional: Combined Types (Example) ---
// Sometimes it's useful to have types that include related data

export interface ChoreAssignmentWithChore extends ChoreAssignment {
	chores: Chore | null; // Assuming 'chores' is the relationship name in your query
}

export interface ChoreAssignmentWithDetails extends ChoreAssignment {
	chores: Chore | null;
	assigned_user: Profile | null; // Assuming 'profiles' table linked via assigned_user_id
}

export interface ItemWithDetails extends Item {
	added_by: Profile | null; // Assuming 'profiles' linked via added_by_id
	completed_by: Profile | null; // Assuming 'profiles' linked via completed_by_id
	price_added_by: Profile | null; // Assuming 'profiles' linked via price_added_by_id
}

// Add other combined types as needed for your specific data fetching patterns
